import { invoke, resolveView } from "./resolution.ts";
import View, { popLocals, pushLocals, type Locals } from "./View.ts";

export default class JSView extends View {
    public override onMetadata() {
        const res = [];

        for (const match of this.src.matchAll(/^(?:#!.+?\n)?\s*\/\*---\n(.+)\n---\*\/$/g)) {
            res.push(match[1]);
        }

        return res.join("\n");
    }

    public override onRender(locals: Locals = {}, full = false): string {
        locals = this.locals(locals);
        let res: any, text;

        try {
            pushLocals(locals);
            res = invoke(this, this.path);
        }
        finally {
            popLocals();
        }

        if (typeof res === "object" && res != null && typeof res[Symbol.iterator] === "function") {
            res = res[Symbol.iterator]();
            const parts = [];

            while (true) {
                const el = res.next();
                if (el.value != null) parts.push(el.value);
                if (el.done) break;
            }

            text = parts.join("");
        }
        else {
            text = String(res ?? "");
        }

        if (full && locals.page) {
            const parent = resolveView(String(locals.page), false, true);
            return parent?.render({ ...locals, text }, full) ?? text;
        }
        else return text;
    }
}